c-----------------------------------------------------------------------
c
c     This file is part of the Test Set for IVP solvers
c     http://www.dm.uniba.it/~testset/
c
c        *** DO NOT DISTRIBUTE THIS FILE***
c
c     for CWI use only
c
c     this is used in $(TOPDIR)/diagrams/output2gp
c     to produce sensible ytics for logscale y and yrange
c     (gnuplot does not handle this nicely)
c
c     $Id: ranges.f,v 1.3 2001/09/06 14:12:15 mazzia Exp $
c
c-----------------------------------------------------------------------
      program ranges
      implicit none
      double precision x, y
      double precision xmin, xmax, ymin, ymax
      double precision lmargin, rmargin, bmargin, tmargin, yfudge

      double precision ytics(100), t
      integer nytics
      integer i

      double precision idfloor, idceil
      double precision minexp, maxexp, iexp

c because of portability problems
c (how many backslashes does it take to get a single backslash?)
c this way BACKSL contains a single backslash character

      character*1 BACKSL
      parameter   (BACKSL = '\\\\\\\\\\\\\\\\')

      xmin = 16d0
      xmax = 0d0
      ymin = 36000d0
      ymax = 0d0

   10 read (*,*,end=20) x,y
         xmin = min(x,xmin)
         xmax = max(x,xmax)
         ymin = min(y,ymin)
         ymax = max(y,ymax)
         goto 10
   20 continue
      ymin = max(ymin,0.0001d0)

c left and right margin 2% of width

      lmargin = 0.02d0 / (1d0 - 0.02d0+0.02d0) * (xmax-xmin)
      rmargin = 0.02d0 / (1d0 - 0.02d0+0.02d0) * (xmax-xmin)

      write (*,*) 'set xrange [', xmin-lmargin, ':', xmax+rmargin, ']'

c top margin 9% of height this accomodates for 4 solvers in legenda
c bottom margin same as left and right margin
c 2% of width: the factor 13.1/16.9 compensates for aspect ratio

      bmargin = 0.02d0*13.1d0/16.9d0 /
     +          (1d0 - 0.02d0*13.1d0/16.9d0-0.09d0) *
     +          (log10(ymax) - log10(ymin))
      tmargin = 0.09d0 / (1d0 - 0.02d0*13.1d0/16.9d0-0.09d0) *
     +          (log10(ymax) - log10(ymin))

      write(*,*) 'set yrange [',
     +         10d0**(log10(ymin)-bmargin),
     +         ':',
     +         10d0**(log10(ymax)+tmargin),
     +         ']'

c yfudge factor to prevent ytics overstriking the `ymin' and `ymax' ytics
c ytic should be at least 1% of height away from ymin and ymax
c this is an experimental percentage
c 2% is too large, 1% might be too small

      yfudge = 0.01d0 / (1d0 - 0.01d0) * (log10(ymax) - log10(ymin))

      maxexp = idfloor(log10(ymax))
      minexp = idceil(log10(ymin))

      nytics = 0

      do 40 iexp=nint(min(minexp,maxexp))-1,nint(max(minexp,maxexp))+1
         do 30 i=1,10
            t = dble(i) * 10d0 ** iexp
            if (t.gt.10d0**(log10(ymin)+yfudge) .and.
     +          t.lt.10d0**(log10(ymax)-yfudge)
     +      ) then
               nytics = nytics + 1
               ytics(nytics) = t
            endif
   30    continue
   40 continue

      write (*,'(''set ytics ('',A1)') BACKSL
      write(*,'(f9.4,'','',A1)') ymin, BACKSL
      do 100 i=1,nytics
         write(*,'(f9.4,'','',A1)') ytics(i),BACKSL
  100 continue
      write(*,'(f9.4,A1)') ymax, BACKSL
      write(*,'('')'')')

      end

      double precision function idfloor(x)
      double precision x
      idfloor = idint(x)
      if (idfloor.gt.x) idfloor = idfloor - 1d0
      return
      end

      double precision function idceil(x)
      double precision x
      idceil = idint(x)
      if (idceil.lt.x) idceil = idceil + 1d0
      return
      end
