c----------------------------------------------------------------------- 
c 
c     This file is part of the Test Set for IVP solvers 
c     http://www.dm.uniba.it/~testset/ 
c 
c        (ODE only) PSODE driver 
c 
c     DISCLAIMER: see 
c     http://www.dm.uniba.it/~testset/disclaimer.php 
c 
c     The most recent version of this source file can be found at 
c     http://www.dm.uniba.it/~testset/src/drivers/psoded.f 
c 
c     This is revision 
c     $Id: psoded.f,v 1.6 2006/10/02 10:19:09 testset Exp $ 
c 
c----------------------------------------------------------------------- 
      program  psoded 
 
      integer    MAXS,     MAXD 
      parameter (MAXS = 4, MAXD = 400) 
 
      double precision  a(MAXS,MAXS),c(MAXS),y(MAXD),v(MAXD,MAXS), 
     +                  aux(MAXD),yref(MAXD),avec(MAXS),d(MAXS), 
     +                  fjm1(MAXD,MAXS),yj(MAXD,MAXS), 
     +                  yjm1(MAXD,MAXS),rjac(MAXD,MAXD), 
     +                  rjlu(MAXD,MAXD,MAXS),dyn(MAXD),yjl(MAXD,MAXS) 
      integer           ipvt(MAXD,MAXS),ijac 
 
      integer neqn,ndisc,mljac,mujac,mlmas,mumas,ind(MAXD) 
      logical numjac,nummas,consis,tolvec 
      double precision yprime(MAXD) 
 
      double precision t(0:100) 
      external odef, odej 
 
      integer cor 
      integer       nsteps,naccpt,nrejct,njac,nlu,nlus,nfe,nfes,nfb,nfbs 
      common /istat/nsteps,naccpt,nrejct,njac,nlu,nlus,nfe,nfes,nfb,nfbs 
 
      double precision rtol(MAXD), atol(MAXD), tol, h0, h, solmax 
      integer k 
 
      real gettim, timtim, cputim 
      external gettim 
 
      character fullnm*40, problm*8, type*3 
      character driver*8, solver*8 
      parameter (driver = 'psoded', solver='PSODE') 
      double precision scd, mescd 
 
      integer i,icount(5) 
      integer ipar(2) 
      double precision rpar(1) 
      common /parcom/ rpar,ipar 
 
      character  fileout*140,  filepath*100 
      character formatout*30,  namefile*100 
      logical printsolout, solref, printout 
      integer nindsol, indsol(MAXD) 
 
c PSODE corrector: 4-stage Radau IIA 
 
      cor=6 
      call correc(cor,MAXS,a,avec,c,k,d) 
 
c----------------------------------------------------------------------- 
c     check the problem definition interface date 
c----------------------------------------------------------------------- 
 
      call chkdat(driver,solver,20060828) 
 
c----------------------------------------------------------------------- 
c     get the problem dependent parameters 
c----------------------------------------------------------------------- 
 
      call prob(fullnm,problm,type, 
     +          neqn,ndisc,t, 
     +          numjac,mljac,mujac, 
     +          nummas,mlmas,mumas, 
     +          ind) 
      if (type.eq.'IDE') then 
         print *, 'PSODED: ERROR: ', 
     +            'PSODE can not solve IDEs' 
         stop 
      elseif (type.eq.'DAE') then 
         print *, 'PSODED: ERROR: ', 
     +            'PSODE can not solve DAEs' 
         stop 
      elseif (type.ne.'ODE') then 
         print *, 'PSODED: ERROR: ', 
     +            'unknown Test Set problem type', type 
         stop 
      endif 
 
      if (numjac) then 
         print *, 'PSODED: ERROR: ', 
     +            'PSODE does not support numerical Jacobians' 
         stop 
      endif 
 
c----------------------------------------------------------------------- 
c     get the initial values 
c----------------------------------------------------------------------- 
 
      call init(neqn,t(0),y,yprime,consis) 
 
c----------------------------------------------------------------------- 
c     read the tolerances, initial step and max(abs(y)) 
c----------------------------------------------------------------------- 
 
      call getinp(driver,problm,solver,fullnm, 
     +            tolvec,rtol,atol,h0,solmax) 
 
      call settolerances(neqn,rtol,atol,tolvec) 
       
      if (tolvec) then 
         print *, 'PSODED: ERROR: ', 
     +            'PSODE does not support non-scalar tolerances' 
         stop 
      endif 
      if (rtol(1).ne.atol(1)) then 
         print *, 'PSODED: WARNING: ', 
     +            'PSODE does not support rtol != atol;', 
     +            'using rtol as atol' 
      endif 
      tol = rtol(1) 
 
      call  setoutput(neqn,solref,printsolout, 
     +                     nindsol,indsol) 
 
 
      if (printsolout) then 
          print *, 'PSODED: WARNING: ', 
     +            'this option is not yet implemented;', 
     +            'we set printsolout  = .false. ' 
          printsolout = .false. 
          
      end if 
 
c----------------------------------------------------------------------- 
c     call of the subroutine PSODE 
c----------------------------------------------------------------------- 
 
      do 10 i=1,5 
         icount(i) = 0 
   10 continue 
 
      timtim = gettim() 
      timtim = gettim() - timtim 
 
      cputim = gettim() 
 
      do 20 i=0,ndisc 
 
         h = h0 
 
         call psode(neqn,MAXS,MAXD,odef,odej,t(i),y,t(i+1),tol,h,k,cor, 
     +              a,c,d,avec,dyn,fjm1,v,aux,ipvt,yj,yjm1, 
     +              yjl,rjac,rjlu,solmax, 
     +              ijac,mljac,mujac) 
 
         icount(1) = icount(1) + nsteps 
         icount(2) = icount(2) + naccpt 
         icount(3) = icount(3) + nfe 
         icount(4) = icount(4) + njac 
         icount(5) = icount(5) + nlu 
 
   20 continue 
 
      cputim = gettim() - cputim - timtim 
 
c----------------------------------------------------------------------- 
c     print numerical solution in endpoint and integration statistics 
c----------------------------------------------------------------------- 
      printout = .true. 
      if (solref) then  
        call solut(neqn,t(ndisc+1),yref)    
        call getscd(mescd,scd,neqn,yref,y,problm,tolvec,atol,rtol, 
     +            printout) 
      else 
        call printsol(neqn,y,problm) 
      end if 
       
      call report( 
     +   driver,problm,solver, 
     +   rtol,atol,h0,solmax, 
     +   icount,cputim,scd,mescd 
     +) 
 
      end 
C======================================================================= 
C     `Test Set for IVP Solvers' ODE wrappers for PSODE 
C======================================================================= 
c 
c     since in PSODE the format of the subroutine containing the 
c     derivative function and its Jacobian differ from the format in the 
c     testset, we transform them 
c 
c----------------------------------------------------------------------- 
      subroutine odef(neqn,t,y,dy) 
      integer neqn 
      double precision t, y(neqn), dy(neqn) 
      integer ierr,ipar(2) 
      double precision rpar(1) 
      common /parcom/ rpar, ipar 
      ierr = 0 
      call feval(neqn,t,y,y,dy,ierr,rpar,ipar) 
      if (ierr.ne.0) then 
         print *, 'PSODED: ERROR: ', 
     +            'PSODE can not handle FEVAL IERR' 
         stop 
      endif 
      return 
      end 
      subroutine odej(neqn,t,y,jac,ldim) 
      integer neqn, ldim 
      double precision t, y(neqn), jac(ldim,neqn) 
      integer ierr,ipar(2) 
      double precision rpar(1) 
      common /parcom/ rpar,ipar 
      ierr = 0 
      call jeval(ldim,neqn,t,y,y,jac,ierr,rpar,ipar) 
      if (ierr.ne.0) then 
         print *, 'PSODED: ERROR: ', 
     +            'PSODE can not handle JEVAL IERR' 
         stop 
      endif 
      return 
      end 
